from math import pi, sin, cos
import optparse
from scipy.signal import ZerosPolesGain
from scipy.signal import ellip

# Mark Tillotson, 2017-07-28
# Digital filter example
# butterworth band pass filter

from utils import quantize
from plotting import plot_response, plot_everything
from propeller import compile_propeller_filter


parser = optparse.OptionParser ()
parser.add_option ('--bits', action = 'store', type = int, default = 16)
parser.add_option ('--spin', action = 'store', default = None)
options, args = parser.parse_args ()

bits = options.bits
assert 4 <= bits <= 24

print ("Spin %s" % options.spin)

fs = 48000
nyquist = 0.5 * fs
cut1 = 8000.0
cut2 = 10000.0
stop_band_dB = 120
ripple_dB = 0.1

z, p, g = ellip (8, ripple_dB, stop_band_dB, [cut1/nyquist, cut2/nyquist], btype = 'bandpass', analog = False, output = 'zpk')

resp = ZerosPolesGain (z, p, g)
zeros = resp.zeros
poles = resp.poles
gain = resp.gain
zeros = [ quantize (zero, bits) for zero in zeros ]
poles = [ quantize (pole, bits) for pole in poles ]
resp = ZerosPolesGain (zeros, poles, gain)

plot_everything (resp, fs, floor = -140)
plot_response (resp, fs, logarithmic = False, floor = -140)
plot_response (resp, fs, logarithmic = True, floor = -140)


print ("Spin %s" % options.spin)
if options.spin is not None:
    compile_propeller_filter (resp, options.spin, options.bits)

"""
////////////////////////////////////////////////////////////////////////////////////////////
//                                TERMS OF USE: MIT License
////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this 
// software and associated documentation files (the "Software"), to deal in the Software 
// without restriction, including without limitation the rights to use, copy, modify, merge,
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit 
// persons to whom the Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
////////////////////////////////////////////////////////////////////////////////////////////
"""
